@@ -1,13 +1,12 @@
+use indexmap::IndexMap;
+use itertools::Itertools;
+use serde::{Deserialize, Deserializer};
 use std::borrow::Cow;
 use std::collections::{BTreeMap, Bound};
 use std::ffi::OsStr;
-use std::fmt::Display;
 use std::fmt::Write;
 use std::path::{Path, PathBuf};
 use std::str::{self, FromStr};
-
-use itertools::Itertools;
-use serde::{Deserialize, Deserializer};
 use tracing::{debug, trace, warn};
 use version_ranges::Ranges;
 use walkdir::WalkDir;
@@ -19,7 +18,7 @@ use uv_pep440::{Version, VersionSpecifiers};
 use uv_pep508::{
     ExtraOperator, MarkerExpression, MarkerTree, MarkerValueExtra, Requirement, VersionOrUrl,
 };
-use uv_pypi_types::{Metadata23, VerbatimParsedUrl};
+use uv_pypi_types::{Keywords, Metadata23, ProjectUrls, VerbatimParsedUrl};
 
 use crate::serde_verbatim::SerdeVerbatim;
 use crate::{BuildBackendSettings, Error, error_on_venv};
@@ -27,6 +26,10 @@ use crate::{BuildBackendSettings, Error, error_on_venv};
 /// By default, we ignore generated python files.
 pub(crate) const DEFAULT_EXCLUDES: &[&str] = &["__pycache__", "*.pyc", "*.pyo"];
 
+/// No breaking changes were introduced to the uv build backend since these releases, so we can use
+/// the fast path for them too.
+const COMPATIBLE_VERSIONS: &[&str] = &["0.9.30"];
+
 #[derive(Debug, Error)]
 pub enum ValidationError {
     /// The spec isn't clear about what the values in that field would be, and we only support the
@@ -67,8 +70,39 @@ pub enum ValidationError {
     LicenseFileNotUtf8(String),
 }
 
+/// The project is not compatible with a direct uv build.
+///
+/// Displays a half-sentence with a reason why it isn't, format into a full sentence for using it
+/// as an error.
+#[derive(Debug, Error)]
+pub enum DirectBuildIncompatibility {
+    #[error("its `pyproject.toml` failed to parse: {0}")]
+    PyprojectToml(String),
+    #[error("`build_system.build-backend` is not `uv_build`, but `{0}`")]
+    WrongBackend(String),
+    #[error("`build-system.requires` is not exactly `uv_build`, but `{0}`")]
+    MultipleRequires(String),
+    #[error("`build-system.requires` is not `uv_build`, but `{0}`")]
+    WrongPackage(PackageName),
+    #[error("`build_system.requires` uses a URL requirement")]
+    UrlRequirement,
+    #[error("`uv_build{0}` is not a known compatible range")]
+    IncompatibleRange(VersionSpecifiers),
+}
+
 /// Check if the build backend is matching the currently running uv version.
-pub fn check_direct_build(source_tree: &Path, name: impl Display) -> bool {
+///
+/// Example table compatible with uv 0.4.21:
+///
+/// ```toml
+/// [build-system]
+/// requires = ["uv_build>=0.4.15,<0.5.0"]
+/// build-backend = "uv_build"
+/// ```
+pub fn check_direct_build(
+    source_tree: &Path,
+    uv_version: &str,
+) -> Result<(), DirectBuildIncompatibility> {
     #[derive(Deserialize)]
     #[serde(rename_all = "kebab-case")]
     struct PyProjectToml {
@@ -83,31 +117,65 @@ pub fn check_direct_build(source_tree: &Path, name: impl Display) -> bool {
             }) {
             Ok(pyproject_toml) => pyproject_toml,
             Err(err) => {
-                debug!(
-                    "Not using uv build backend direct build for source tree `{name}`, \
-                    failed to parse pyproject.toml: {err}"
-                );
-                return false;
+                return Err(DirectBuildIncompatibility::PyprojectToml(err));
             }
         };
-    match pyproject_toml
-        .build_system
-        .check_build_system(uv_version::version())
-        .as_slice()
-    {
-        // No warnings -> match
-        [] => true,
-        // Any warning -> no match
-        [first, others @ ..] => {
-            debug!(
-                "Not using uv build backend direct build of `{name}`, pyproject.toml does not match: {first}"
-            );
-            for other in others {
-                trace!("Further uv build backend direct build of `{name}` mismatch: {other}");
+
+    if pyproject_toml.build_system.build_backend.as_deref() != Some("uv_build") {
+        return Err(DirectBuildIncompatibility::WrongBackend(
+            pyproject_toml
+                .build_system
+                .build_backend
+                .clone()
+                .unwrap_or_default(),
+        ));
+    }
+
+    let compatible: Vec<Version> = COMPATIBLE_VERSIONS
+        .iter()
+        .chain([&uv_version])
+        .map(|version| {
+            Version::from_str(version).expect("hardcoded version is not PEP 440 compliant")
+        })
+        .collect();
+
+    let [uv_requirement] = &pyproject_toml.build_system.requires.as_slice() else {
+        return Err(DirectBuildIncompatibility::MultipleRequires(
+            pyproject_toml
+                .build_system
+                .requires
+                .iter()
+                .map(ToString::to_string)
+                .join("`, `"),
+        ));
+    };
+    if uv_requirement.name.as_str() != "uv-build" {
+        return Err(DirectBuildIncompatibility::WrongPackage(
+            uv_requirement.name.clone(),
+        ));
+    }
+    match &uv_requirement.version_or_url {
+        None => {
+            // If the user doesn't set any upper bound, we don't help them by not using the fast
+            // path, their build may equally fail if the index version of `uv_build`.
+        }
+        Some(VersionOrUrl::Url(_)) => {
+            // We can't validate the url.
+            return Err(DirectBuildIncompatibility::UrlRequirement);
+        }
+        Some(VersionOrUrl::VersionSpecifier(specifier)) => {
+            // If the user doesn't set an upper bound, we don't help them by not using the fast
+            // path, their build may equally fail if the index version of `uv_build`, so we allow
+            // missing upper bounds.
+            if !compatible.iter().any(|version| specifier.contains(version)) {
+                return Err(DirectBuildIncompatibility::IncompatibleRange(
+                    specifier.clone(),
+                ));
             }
-            false
         }
     }
+
+    Ok(())
 }
 
 /// A package name as provided in a `pyproject.toml`.
@@ -353,13 +421,7 @@ impl PyProjectToml {
         let (license, license_expression, license_files) = self.license_metadata(root)?;
 
         // TODO(konsti): https://peps.python.org/pep-0753/#label-normalization (Draft)
-        let project_urls = self
-            .project
-            .urls
-            .iter()
-            .flatten()
-            .map(|(key, value)| format!("{key}, {value}"))
-            .collect();
+        let project_urls = ProjectUrls::new(self.project.urls.clone().unwrap_or_default());
 
         let extras = self
             .project
@@ -404,11 +466,7 @@ impl PyProjectToml {
             summary,
             description,
             description_content_type,
-            keywords: self
-                .project
-                .keywords
-                .as_ref()
-                .map(|keywords| keywords.join(",")),
+            keywords: self.project.keywords.clone().map(Keywords::new),
             home_page: None,
             download_url: None,
             author,
@@ -438,7 +496,7 @@ impl PyProjectToml {
     }
 
     /// Parse and validate the old (PEP 621) and new (PEP 639) license files.
-    #[allow(clippy::type_complexity)]
+    #[expect(clippy::type_complexity)]
     fn license_metadata(
         &self,
         root: &Path,
@@ -699,7 +757,7 @@ struct Project {
     /// PyPI shows all URLs with their name. For some known patterns, they add favicons.
     /// main: <https://github.com/pypi/warehouse/blob/main/warehouse/templates/packaging/detail.html>
     /// archived: <https://github.com/pypi/warehouse/blob/e3bd3c3805ff47fff32b67a899c1ce11c16f3c31/warehouse/templates/packaging/detail.html>
-    urls: Option<BTreeMap<String, String>>,
+    urls: Option<IndexMap<String, String>>,
     /// The console entrypoints of the project.
     ///
     /// The key of the table is the name of the entry point and the value is the object reference.
@@ -848,7 +906,7 @@ impl BuildSystem {
         let mut warnings = Vec::new();
         if self.build_backend.as_deref() != Some("uv_build") {
             warnings.push(format!(
-                r#"The value for `build_system.build-backend` should be `"uv_build"`, not `"{}"`"#,
+                r#"`build_system.build-backend` was expected to be `"uv_build"`, not `"{}"`"#,
                 self.build_backend.clone().unwrap_or_default()
             ));
         }
@@ -858,19 +916,18 @@ impl BuildSystem {
         let next_minor = uv_version.release().get(1).copied().unwrap_or_default() + 1;
         let next_breaking = Version::new([0, next_minor]);
 
-        let expected = || {
-            format!(
-                "Expected a single uv requirement in `build-system.requires`, found `{}`",
-                toml::to_string(&self.requires).unwrap_or_default()
-            )
-        };
-
         let [uv_requirement] = &self.requires.as_slice() else {
-            warnings.push(expected());
+            warnings.push(format!(
+                "Expected `build-system.requires` to contain only `uv_build`, found `{}`",
+                self.requires.iter().map(ToString::to_string).join("`, `")
+            ));
             return warnings;
         };
         if uv_requirement.name.as_str() != "uv-build" {
-            warnings.push(expected());
+            warnings.push(format!(
+                "Expected `build-system.requires` to be `uv_build`, found `{}`",
+                self.requires.iter().map(ToString::to_string).join("`, `")
+            ));
             return warnings;
         }
         let bounded = match &uv_requirement.version_or_url {
@@ -960,7 +1017,7 @@ mod tests {
         let temp_dir = TempDir::new().unwrap();
 
         let metadata = pyproject_toml.to_metadata(temp_dir.path()).unwrap();
-        assert_snapshot!(metadata.core_metadata_format(), @r"
+        assert_snapshot!(metadata.core_metadata_format(), @"
         Metadata-Version: 2.3
         Name: Hello-World
         Version: 0.1.0
@@ -1044,7 +1101,7 @@ mod tests {
         let pyproject_toml: PyProjectToml = toml::from_str(contents).unwrap();
         let metadata = pyproject_toml.to_metadata(temp_dir.path()).unwrap();
 
-        assert_snapshot!(metadata.core_metadata_format(), @r###"
+        assert_snapshot!(metadata.core_metadata_format(), @r#"
         Metadata-Version: 2.3
         Name: hello-world
         Version: 0.1.0
@@ -1077,9 +1134,9 @@ mod tests {
         # Foo
 
         This is the foo library.
-        "###);
+        "#);
 
-        assert_snapshot!(pyproject_toml.to_entry_points().unwrap().unwrap(), @r###"
+        assert_snapshot!(pyproject_toml.to_entry_points().unwrap().unwrap(), @"
         [console_scripts]
         foo = foo.cli:__main__
 
@@ -1088,8 +1145,7 @@ mod tests {
 
         [bar_group]
         foo-bar = foo:bar
-
-        "###);
+        ");
     }
 
     #[test]
@@ -1138,7 +1194,7 @@ mod tests {
         let pyproject_toml: PyProjectToml = toml::from_str(contents).unwrap();
         let metadata = pyproject_toml.to_metadata(temp_dir.path()).unwrap();
 
-        assert_snapshot!(metadata.core_metadata_format(), @r"
+        assert_snapshot!(metadata.core_metadata_format(), @"
         Metadata-Version: 2.3
         Name: hello-world
         Version: 0.1.0
@@ -1230,7 +1286,7 @@ mod tests {
         let pyproject_toml: PyProjectToml = toml::from_str(contents).unwrap();
         let metadata = pyproject_toml.to_metadata(temp_dir.path()).unwrap();
 
-        assert_snapshot!(metadata.core_metadata_format(), @r###"
+        assert_snapshot!(metadata.core_metadata_format(), @r#"
         Metadata-Version: 2.3
         Name: hello-world
         Version: 0.1.0
@@ -1270,9 +1326,9 @@ mod tests {
         # Foo
 
         This is the foo library.
-        "###);
+        "#);
 
-        assert_snapshot!(pyproject_toml.to_entry_points().unwrap().unwrap(), @r###"
+        assert_snapshot!(pyproject_toml.to_entry_points().unwrap().unwrap(), @"
         [console_scripts]
         foo = foo.cli:__main__
 
@@ -1281,8 +1337,7 @@ mod tests {
 
         [bar_group]
         foo-bar = foo:bar
-
-        "###);
+        ");
     }
 
     #[test]
@@ -1309,7 +1364,7 @@ mod tests {
         let pyproject_toml: PyProjectToml = toml::from_str(contents).unwrap();
         assert_snapshot!(
             pyproject_toml.check_build_system("0.4.15+test").join("\n"),
-            @r###"`build_system.requires = ["uv_build"]` is missing an upper bound on the `uv_build` version such as `<0.5`. Without bounding the `uv_build` version, the source distribution will break when a future, breaking version of `uv_build` is released."###
+            @r#"`build_system.requires = ["uv_build"]` is missing an upper bound on the `uv_build` version such as `<0.5`. Without bounding the `uv_build` version, the source distribution will break when a future, breaking version of `uv_build` is released."#
         );
     }
 
@@ -1327,7 +1382,7 @@ mod tests {
         let pyproject_toml: PyProjectToml = toml::from_str(contents).unwrap();
         assert_snapshot!(
             pyproject_toml.check_build_system("0.4.15+test").join("\n"),
-            @"Expected a single uv requirement in `build-system.requires`, found ``"
+            @"Expected `build-system.requires` to contain only `uv_build`, found `uv-build>=0.4.15,<0.5.0`, `wheel`"
         );
     }
 
@@ -1345,7 +1400,7 @@ mod tests {
         let pyproject_toml: PyProjectToml = toml::from_str(contents).unwrap();
         assert_snapshot!(
             pyproject_toml.check_build_system("0.4.15+test").join("\n"),
-            @"Expected a single uv requirement in `build-system.requires`, found ``"
+            @"Expected `build-system.requires` to be `uv_build`, found `setuptools`"
         );
     }
 
@@ -1363,7 +1418,7 @@ mod tests {
         let pyproject_toml: PyProjectToml = toml::from_str(contents).unwrap();
         assert_snapshot!(
             pyproject_toml.check_build_system("0.4.15+test").join("\n"),
-            @r###"The value for `build_system.build-backend` should be `"uv_build"`, not `"setuptools"`"###
+            @r#"`build_system.build-backend` was expected to be `"uv_build"`, not `"setuptools"`"#
         );
     }
 
@@ -1376,11 +1431,11 @@ mod tests {
             .to_metadata(Path::new("/do/not/read"))
             .unwrap();
 
-        assert_snapshot!(metadata.core_metadata_format(), @r###"
+        assert_snapshot!(metadata.core_metadata_format(), @"
         Metadata-Version: 2.3
         Name: hello-world
         Version: 0.1.0
-        "###);
+        ");
     }
 
     #[test]
@@ -1433,7 +1488,7 @@ mod tests {
             .unwrap()
             .to_metadata(Path::new("/do/not/read"))
             .unwrap_err();
-        assert_snapshot!(format_err(err), @r"
+        assert_snapshot!(format_err(err), @"
         Invalid project metadata
           Caused by: `project.description` must be a single line
         ");
@@ -1451,7 +1506,7 @@ mod tests {
             .unwrap()
             .to_metadata(Path::new("/do/not/read"))
             .unwrap_err();
-        assert_snapshot!(format_err(err), @r"
+        assert_snapshot!(format_err(err), @"
         Invalid project metadata
           Caused by: When `project.license-files` is defined, `project.license` must be an SPDX expression string
         ");
@@ -1467,12 +1522,12 @@ mod tests {
             .unwrap()
             .to_metadata(Path::new("/do/not/read"))
             .unwrap();
-        assert_snapshot!(metadata.core_metadata_format(), @r###"
+        assert_snapshot!(metadata.core_metadata_format(), @"
         Metadata-Version: 2.4
         Name: hello-world
         Version: 0.1.0
         License-Expression: MIT OR Apache-2.0
-        "###);
+        ");
     }
 
     #[test]
@@ -1486,7 +1541,7 @@ mod tests {
             .to_metadata(Path::new("/do/not/read"))
             .unwrap_err();
         // TODO(konsti): We mess up the indentation in the error.
-        assert_snapshot!(format_err(err), @r"
+        assert_snapshot!(format_err(err), @"
         Invalid project metadata
           Caused by: `project.license` is not a valid SPDX expression: MIT XOR Apache-2
           Caused by: MIT XOR Apache-2
@@ -1505,7 +1560,7 @@ mod tests {
             .unwrap()
             .to_metadata(Path::new("/do/not/read"))
             .unwrap_err();
-        assert_snapshot!(format_err(err), @r"
+        assert_snapshot!(format_err(err), @"
         Invalid project metadata
           Caused by: Dynamic metadata is not supported
         ");
@@ -1548,4 +1603,153 @@ mod tests {
         });
         assert_snapshot!(script_error(&contents), @"Use `project.gui-scripts` instead of `project.entry-points.gui_scripts`");
     }
+
+    #[test]
+    fn check_direct_build_ok() {
+        let temp_dir = TempDir::new().unwrap();
+        fs_err::write(
+            temp_dir.path().join("pyproject.toml"),
+            indoc! {r#"
+                [project]
+                name = "hello-world"
+                version = "0.1.0"
+
+                [build-system]
+                requires = ["uv_build>=0.10.0,<0.11"]
+                build-backend = "uv_build"
+            "#},
+        )
+        .unwrap();
+        check_direct_build(temp_dir.path(), "0.10.0").unwrap();
+    }
+
+    #[test]
+    fn check_direct_build_parse_error() {
+        let temp_dir = TempDir::new().unwrap();
+        fs_err::write(
+            temp_dir.path().join("pyproject.toml"),
+            "invalid toml >>>>>>>",
+        )
+        .unwrap();
+        assert_snapshot!(
+            check_direct_build(temp_dir.path(), "0.10.0").unwrap_err(),
+            @r#"
+            its `pyproject.toml` failed to parse: TOML parse error at line 1, column 9
+              |
+            1 | invalid toml >>>>>>>
+              |         ^
+            key with no value, expected `=`
+            "#
+        );
+    }
+
+    #[test]
+    fn check_direct_build_wrong_backend() {
+        let temp_dir = TempDir::new().unwrap();
+        fs_err::write(
+            temp_dir.path().join("pyproject.toml"),
+            indoc! {r#"
+                [project]
+                name = "hello-world"
+                version = "0.1.0"
+
+                [build-system]
+                requires = ["setuptools"]
+                build-backend = "setuptools"
+            "#},
+        )
+        .unwrap();
+        assert_snapshot!(
+            check_direct_build(temp_dir.path(), "0.10.0").unwrap_err(),
+            @"`build_system.build-backend` is not `uv_build`, but `setuptools`"
+        );
+    }
+
+    #[test]
+    fn check_direct_build_multiple_requires() {
+        let temp_dir = TempDir::new().unwrap();
+        fs_err::write(
+            temp_dir.path().join("pyproject.toml"),
+            indoc! {r#"
+                [project]
+                name = "hello-world"
+                version = "0.1.0"
+
+                [build-system]
+                requires = ["uv_build>=0.10.0,<0.11", "wheel"]
+                build-backend = "uv_build"
+            "#},
+        )
+        .unwrap();
+        assert_snapshot!(
+            check_direct_build(temp_dir.path(), "0.10.0").unwrap_err(),
+            @"`build-system.requires` is not exactly `uv_build`, but `uv-build>=0.10.0,<0.11`, `wheel`"
+        );
+    }
+
+    #[test]
+    fn check_direct_build_wrong_package() {
+        let temp_dir = TempDir::new().unwrap();
+        fs_err::write(
+            temp_dir.path().join("pyproject.toml"),
+            indoc! {r#"
+                [project]
+                name = "hello-world"
+                version = "0.1.0"
+
+                [build-system]
+                requires = ["setuptools>=70"]
+                build-backend = "uv_build"
+            "#},
+        )
+        .unwrap();
+        assert_snapshot!(
+            check_direct_build(temp_dir.path(), "0.10.0").unwrap_err(),
+            @"`build-system.requires` is not `uv_build`, but `setuptools`"
+        );
+    }
+
+    #[test]
+    fn check_direct_build_url_requirement() {
+        let temp_dir = TempDir::new().unwrap();
+        fs_err::write(
+            temp_dir.path().join("pyproject.toml"),
+            indoc! {r#"
+                [project]
+                name = "hello-world"
+                version = "0.1.0"
+
+                [build-system]
+                requires = ["uv_build @ https://example.com/uv_build-0.10.0-py3-none-any.whl"]
+                build-backend = "uv_build"
+            "#},
+        )
+        .unwrap();
+        assert_snapshot!(
+            check_direct_build(temp_dir.path(), "0.10.0").unwrap_err(),
+            @"`build_system.requires` uses a URL requirement"
+        );
+    }
+
+    #[test]
+    fn check_direct_build_incompatible_range() {
+        let temp_dir = TempDir::new().unwrap();
+        fs_err::write(
+            temp_dir.path().join("pyproject.toml"),
+            indoc! {r#"
+                [project]
+                name = "hello-world"
+                version = "0.1.0"
+
+                [build-system]
+                requires = ["uv_build>=0.5.0,<0.6"]
+                build-backend = "uv_build"
+            "#},
+        )
+        .unwrap();
+        assert_snapshot!(
+            check_direct_build(temp_dir.path(), "0.10.0").unwrap_err(),
+            @"`uv_build>=0.5.0, <0.6` is not a known compatible range"
+        );
+    }
 }
