@@ -37,6 +37,8 @@ pub enum PreviewFeature {
     AdjustUlimit = 1 << 22,
     SpecialCondaEnvNames = 1 << 23,
     RelocatableEnvsDefault = 1 << 24,
+    PublishRequireNormalized = 1 << 25,
+    Audit = 1 << 26,
 }
 
 impl PreviewFeature {
@@ -68,6 +70,8 @@ impl PreviewFeature {
             Self::AdjustUlimit => "adjust-ulimit",
             Self::SpecialCondaEnvNames => "special-conda-env-names",
             Self::RelocatableEnvsDefault => "relocatable-envs-default",
+            Self::PublishRequireNormalized => "publish-require-normalized",
+            Self::Audit => "audit",
         }
     }
 }
@@ -112,6 +116,8 @@ impl FromStr for PreviewFeature {
             "adjust-ulimit" => Self::AdjustUlimit,
             "special-conda-env-names" => Self::SpecialCondaEnvNames,
             "relocatable-envs-default" => Self::RelocatableEnvsDefault,
+            "publish-require-normalized" => Self::PublishRequireNormalized,
+            "audit" => Self::Audit,
             _ => return Err(PreviewFeatureParseError),
         })
     }
@@ -154,10 +160,20 @@ impl Preview {
         Self::new(preview_features)
     }
 
-    /// Check if a single feature is enabled
+    /// Check if a single feature is enabled.
     pub fn is_enabled(&self, flag: PreviewFeature) -> bool {
         self.flags.contains(flag)
     }
+
+    /// Check if all preview feature rae enabled.
+    pub fn all_enabled(&self) -> bool {
+        self.flags.is_all()
+    }
+
+    /// Check if any preview feature is enabled.
+    pub fn any_enabled(&self) -> bool {
+        !self.flags.is_empty()
+    }
 }
 
 impl Display for Preview {
@@ -343,5 +359,9 @@ mod tests {
             PreviewFeature::RelocatableEnvsDefault.as_str(),
             "relocatable-envs-default"
         );
+        assert_eq!(
+            PreviewFeature::PublishRequireNormalized.as_str(),
+            "publish-require-normalized"
+        );
     }
 }
