@@ -33,6 +33,7 @@ use uv_pep508::{MarkerEnvironment, RequirementOrigin, VerbatimUrl};
 use uv_platform_tags::Tags;
 use uv_preview::Preview;
 use uv_pypi_types::{Conflicts, ResolverMarkerEnvironment};
+use uv_python::managed::{ManagedPythonInstallation, PythonMinorVersionLink};
 use uv_python::{PythonEnvironment, PythonInstallation};
 use uv_requirements::{
     GroupsSpecification, LookaheadResolver, NamedRequirementsResolver, RequirementsSource,
@@ -127,7 +128,7 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
     flat_index: &FlatIndex,
     index: &InMemoryIndex,
     build_dispatch: &BuildDispatch<'_>,
-    concurrency: Concurrency,
+    concurrency: &Concurrency,
     options: Options,
     logger: Box<dyn ResolveLogger>,
     printer: Printer,
@@ -155,7 +156,11 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
                 NamedRequirementsResolver::new(
                     hasher,
                     index,
-                    DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
+                    DistributionDatabase::new(
+                        client,
+                        build_dispatch,
+                        concurrency.downloads_semaphore.clone(),
+                    ),
                 )
                 .with_reporter(Arc::new(ResolverReporter::from(printer)))
                 .resolve(unnamed.into_iter())
@@ -169,7 +174,11 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
                 extras,
                 hasher,
                 index,
-                DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
+                DistributionDatabase::new(
+                    client,
+                    build_dispatch,
+                    concurrency.downloads_semaphore.clone(),
+                ),
             )
             .with_reporter(Arc::new(ResolverReporter::from(printer)))
             .resolve(source_trees.iter())
@@ -277,7 +286,11 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
                 NamedRequirementsResolver::new(
                     hasher,
                     index,
-                    DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
+                    DistributionDatabase::new(
+                        client,
+                        build_dispatch,
+                        concurrency.downloads_semaphore.clone(),
+                    ),
                 )
                 .with_reporter(Arc::new(ResolverReporter::from(printer)))
                 .resolve(unnamed.into_iter())
@@ -308,7 +321,11 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
                 &overrides,
                 hasher,
                 index,
-                DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
+                DistributionDatabase::new(
+                    client,
+                    build_dispatch,
+                    concurrency.downloads_semaphore.clone(),
+                ),
             )
             .with_reporter(Arc::new(ResolverReporter::from(printer)))
             .resolve(&resolver_env)
@@ -356,7 +373,11 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
             hasher,
             build_dispatch,
             installed_packages,
-            DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
+            DistributionDatabase::new(
+                client,
+                build_dispatch,
+                concurrency.downloads_semaphore.clone(),
+            ),
         )?
         .with_reporter(Arc::new(reporter));
 
@@ -543,7 +564,7 @@ pub(crate) async fn install(
     tags: &Tags,
     client: &RegistryClient,
     in_flight: &InFlight,
-    concurrency: Concurrency,
+    concurrency: &Concurrency,
     build_dispatch: &BuildDispatch<'_>,
     cache: &Cache,
     venv: &PythonEnvironment,
@@ -684,7 +705,7 @@ pub(crate) async fn install(
     }
 
     if compile {
-        compile_bytecode(venv, &concurrency, cache, printer).await?;
+        compile_bytecode(venv, concurrency, cache, printer).await?;
     }
 
     // Construct a summary of the changes made to the environment.
@@ -721,7 +742,7 @@ async fn execute_plan(
     tags: &Tags,
     client: &RegistryClient,
     in_flight: &InFlight,
-    concurrency: Concurrency,
+    concurrency: &Concurrency,
     build_dispatch: &BuildDispatch<'_>,
     cache: &Cache,
     venv: &PythonEnvironment,
@@ -748,7 +769,11 @@ async fn execute_plan(
             tags,
             hasher,
             build_options,
-            DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
+            DistributionDatabase::new(
+                client,
+                build_dispatch,
+                concurrency.downloads_semaphore.clone(),
+            ),
         )
         .with_reporter(Arc::new(
             PrepareReporter::from(printer).with_length(remote.len() as u64),
@@ -900,13 +925,26 @@ pub(crate) fn report_target_environment(
     cache: &Cache,
     printer: Printer,
 ) -> Result<(), Error> {
+    // Resolve minor-version link directories (e.g., `cpython-3.12` → `cpython-3.12.12`).
+    // On Windows, junction points aren't resolved by the interpreter's `sys.prefix`, so we
+    // use the target directory from the minor-version link to display the actual installation.
+    // This only applies to managed installations, not virtual environments.
+    let root = if env.interpreter().is_virtualenv() {
+        env.root().to_path_buf()
+    } else {
+        ManagedPythonInstallation::try_from_interpreter(env.interpreter())
+            .and_then(|installation| PythonMinorVersionLink::from_installation(&installation))
+            .map(|link| link.target_directory)
+            .unwrap_or_else(|| env.root().to_path_buf())
+    };
+
     let message = format!(
         "Using Python {} environment at: {}",
         env.interpreter().python_version(),
-        env.root().user_display()
+        root.user_display()
     );
 
-    let Ok(target) = std::path::absolute(env.root()) else {
+    let Ok(target) = std::path::absolute(&root) else {
         debug!("{}", message);
         return Ok(());
     };
