@@ -393,7 +393,7 @@ pub(crate) async fn pip_sync(
         exclude_newer.clone(),
         sources.clone(),
         WorkspaceCache::default(),
-        concurrency,
+        concurrency.clone(),
         preview,
     );
 
@@ -486,7 +486,7 @@ pub(crate) async fn pip_sync(
             &flat_index,
             state.index(),
             &build_dispatch,
-            concurrency,
+            &concurrency,
             options,
             Box::new(DefaultResolveLogger),
             printer,
@@ -531,7 +531,7 @@ pub(crate) async fn pip_sync(
         exclude_newer.clone(),
         sources,
         WorkspaceCache::default(),
-        concurrency,
+        concurrency.clone(),
         preview,
     );
 
@@ -549,7 +549,7 @@ pub(crate) async fn pip_sync(
         &tags,
         &client,
         state.in_flight(),
-        concurrency,
+        &concurrency,
         &build_dispatch,
         &cache,
         &environment,
