@@ -283,7 +283,7 @@ pub(crate) async fn sync(
                 Box::new(DefaultResolveLogger),
                 Box::new(DefaultInstallLogger),
                 installer_metadata,
-                concurrency,
+                &concurrency,
                 cache,
                 workspace_cache.clone(),
                 dry_run,
@@ -350,7 +350,7 @@ pub(crate) async fn sync(
             &client_builder,
             &state,
             Box::new(DefaultResolveLogger),
-            concurrency,
+            &concurrency,
             cache,
             &workspace_cache,
             printer,
@@ -410,7 +410,7 @@ pub(crate) async fn sync(
         &state,
         Box::new(DefaultInstallLogger),
         installer_metadata,
-        concurrency,
+        &concurrency,
         cache,
         workspace_cache,
         dry_run,
@@ -613,7 +613,7 @@ pub(super) async fn do_sync(
     state: &PlatformState,
     logger: Box<dyn InstallLogger>,
     installer_metadata: bool,
-    concurrency: Concurrency,
+    concurrency: &Concurrency,
     cache: &Cache,
     workspace_cache: WorkspaceCache,
     dry_run: DryRun,
@@ -815,7 +815,7 @@ pub(super) async fn do_sync(
         exclude_newer.clone(),
         sources.clone(),
         workspace_cache.clone(),
-        concurrency,
+        concurrency.clone(),
         preview,
     );
 
