@@ -396,6 +396,9 @@ pub(crate) async fn install(
     )
     .await?;
 
+    // Resolve the excludes.
+    let excludes = spec.excludes.clone();
+
     // Resolve the build constraints.
     let build_constraints: Vec<Requirement> =
         operations::read_constraints(build_constraints, &client_builder)
@@ -753,6 +756,7 @@ pub(crate) async fn install(
         requirements,
         constraints,
         overrides,
+        excludes,
         build_constraints,
         printer,
     )?;
