@@ -323,7 +323,7 @@ async fn upgrade_tool(
         Constraints::from_requirements(existing_tool_receipt.build_constraints().iter().cloned());
 
     // Resolve the requirements.
-    let spec = RequirementsSpecification::from_overrides(
+    let spec = RequirementsSpecification::from_excludes(
         existing_tool_receipt.requirements().to_vec(),
         existing_tool_receipt
             .constraints()
@@ -332,6 +332,7 @@ async fn upgrade_tool(
             .cloned()
             .collect(),
         existing_tool_receipt.overrides().to_vec(),
+        existing_tool_receipt.excludes().to_vec(),
     );
 
     // Initialize any shared state.
@@ -446,6 +447,7 @@ async fn upgrade_tool(
             existing_tool_receipt.requirements().to_vec(),
             existing_tool_receipt.constraints().to_vec(),
             existing_tool_receipt.overrides().to_vec(),
+            existing_tool_receipt.excludes().to_vec(),
             existing_tool_receipt.build_constraints().to_vec(),
             printer,
         )?;
